from django.conf import settings

OIDC_AUDIENCE = getattr(settings, "OIDC_AUDIENCE", ["account", "broker"])
OIDC_CONFIG_URL = getattr(settings, "OIDC_CONFIG_URL", None)

# key from KeyCloak; value is user model
OIDC_USER_UPDATE = getattr(settings, "OIDC_USER_UPDATE", True)
OIDC_USER_MODIFIED_FIELD = getattr(settings, "OIDC_USER_MODIFIED_FIELD", "modified_timestamp")
OIDC_TOKEN_MODIFIED_FIELD = getattr(settings, "OIDC_TOKEN_MODIFIED_FIELD", "updated_at")
OIDC_USER_UID = getattr(settings, "OIDC_USER_UID", "kc_id")
OIDC_USER_MAPPING = getattr(
    settings,
    "OIDC_USER_MAPPING",
    {
        "given_name": "first_name",
        "family_name": "last_name",
        "name": "username",
    },
)
OIDC_USER_DEFAULTS = getattr(
    settings,
    "OIDC_USER_DEFAULTS",
    {
        "is_active": True,
    },
)
OIDC_USER_ON_CREATE = getattr(
    settings,
    "OIDC_USER_ON_CREATE",
    None,
)
OIDC_USER_ON_UPDATE = getattr(
    settings,
    "OIDC_USER_ON_UPDATE",
    None,
)

OIDC_CONFIG_ROUTES = getattr(settings, "OIDC_CONFIG_ROUTES", None)



from http import HTTPStatus
from logging import getLogger

from django.http import JsonResponse
from django.utils.deprecation import MiddlewareMixin
from jwt import ExpiredSignatureError

from django_jwt.user import UserHandler
from django_jwt.utils import oidc_handler

log = getLogger(__name__)


class JWTAuthMiddleware(MiddlewareMixin):
    header_key = "HTTP_AUTHORIZATION"

    def process_request(self, request):
        if self.header_key not in request.META:
            return

        auth_header = request.META[self.header_key]
        if not auth_header.startswith("Bearer "):
            return

        # auth part
        raw_token = auth_header[7:]
        try:
            info = oidc_handler.decode_token(raw_token)
            request.user = request._cached_user = UserHandler(info, request, raw_token).get_user()
        except ExpiredSignatureError:
            return JsonResponse(status=HTTPStatus.UNAUTHORIZED.value, data={"detail": "expired token"})
        except UnicodeDecodeError as exc:
            log.error(f"UnicodeDecodeError: {exc}")
        except Exception as exc:
            log.error(f"Unexpected error: {exc}")
            return



import base64
import json
from functools import cache

import jwt
import requests
from jwt.algorithms import ECAlgorithm, RSAAlgorithm

from django_jwt import settings


def get_alg(token: str) -> str:
    header = json.loads(base64.b64decode(token.split(".")[0] + "==="))
    return header["alg"]


class Config:
    def __init__(self):
        self.route = settings.OIDC_CONFIG_ROUTES

    @cache
    def cfg(self, alg: str) -> dict:
        if self.route and alg in self.route:
            return requests.get(self.route[alg]).json()
        return requests.get(settings.OIDC_CONFIG_URL).json()

    @cache
    def get_public_key(self, alg: str) -> str:
        certs_data_response = requests.get(self.cfg(alg)["jwks_uri"])
        certs_data_response.raise_for_status()

        certs_data = certs_data_response.json()
        for key_data in certs_data["keys"]:
            if key_data["alg"] == alg:
                algorithm = RSAAlgorithm if key_data["kty"] == "RSA" else ECAlgorithm
                return algorithm.from_jwk(json.dumps(key_data))


_config = Config()


class OIDCHandler:
    def get_user_info(self, token: str) -> dict:
        alg = get_alg(token)
        url = _config.cfg(alg)["userinfo_endpoint"]
        response = requests.get(url, headers={"Authorization": f"Bearer {token}"})
        response.raise_for_status()
        return response.json()

    def decode_token(self, token: str) -> dict:
        alg = get_alg(token)
        public_key = _config.get_public_key(alg)
        if not public_key:
            raise Exception(f"Public key for {alg} not found")

        return jwt.decode(
            token,
            key=public_key,
            algorithms=[alg],
            audience=settings.OIDC_AUDIENCE,
            options={"verify_aud": False},
        )


oidc_handler = OIDCHandler()



from django.http.response import HttpResponse


def silent_sso_check(request):
    return HttpResponse("<html><body><script>parent.postMessage(location.href, location.origin)</script></body></html>")
